/* Copyright (C) 2001 Free Software Foundation, Inc.
     Contributed by Jakub Jelinek <jakub@redhat.com>.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with the GNU C Library; see the file COPYING.LIB.  If
   not, write to the Free Software Foundation, Inc.,
   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

/* Constants shared between setcontext() and getcontext().  Don't
   install this header file.  */

#define UC_LINK		0
#define __UC_SIGMASK	16
#define UC_M_PC		40
#define UC_M_NPC	48
#define UC_SIGMASK	536
#define SIGMASK_WORDS	16
